use std::ffi::CString;

pub fn use_layers(
    entry: &ash::Entry,
    layers_to_select: Vec<&str>,
) -> Vec<CString> {
    let layers_available = get_layers_available(entry);

    let selected_layers = select_layers(&layers_available, &layers_to_select);

    selected_layers
        .iter()
        .map(|sl| CString::new(sl.clone().as_bytes()).unwrap())
        .collect::<Vec<_>>()
}

fn get_layers_available(entry: &ash::Entry) -> Vec<ash::vk::LayerProperties> {
    unsafe { entry.enumerate_instance_layer_properties().unwrap_or_default() }
}

fn select_layers(
    layers_available: &[ash::vk::LayerProperties],
    layers_to_select: &[&str],
) -> Vec<String> {
    layers_available
        .iter()
        .filter_map(|l| {
            let layer_name = l
                .layer_name_as_c_str()
                .unwrap_or_default()
                .to_string_lossy();
            layers_to_select
                .iter()
                .find(|&&ln| ln == layer_name)
                .map(|_| layer_name.into_owned())
        })
        .collect()
}
