use ash::prelude::VkResult;
use crate::vulkan::VkPhysicalDevice;

pub struct VkSurface {
    surface_loader: ash::khr::surface::Instance,
    surface: ash::vk::SurfaceKHR,
}

impl VkSurface {
    pub fn new(
        surface_loader: ash::khr::surface::Instance,
        surface: ash::vk::SurfaceKHR,
    ) -> Self {
        Self {
            surface_loader,
            surface
        }
    }
    
    pub fn physical_device_queue_supported(&self, physical_device: &VkPhysicalDevice, queue_index: u32) -> VkResult<bool> {
        unsafe {
            self.surface_loader.get_physical_device_surface_support(
                physical_device.handle,
                queue_index,
                self.surface
            )
        }
    }
}
